package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;


public class AjaxCommandButtonTest extends SeleniumTestBase implements RichSeleniumTest {

    public AjaxCommandButtonTest() {
        super("http", "localhost", serverPort);
    }
    
    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
    	super.startSelenium(browser);
    }
    
    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
    	super.stopSelenium();
    }

    @Test
    public void testAjaxCommandButtonComponent() throws Exception {
    	_testAjaxCommandButtonComponent(Templates.SIMPLE);
    	_testAjaxCommandButtonComponent(Templates.DATATABLE);
    	_testAjaxCommandButtonComponent(Templates.MODALPANEL);
    }

    private void _testAjaxCommandButtonComponent(Templates template) {
        renderPage(getTestUrl(), template);
    
    	String parentId = getParentId() + "_form:";
    	
    	String buttonId = parentId + "b1";
    	boolean ajaxSingle = false;
    	boolean immediate = false;
    	writeStatus("Click button 1");
    	setValidation(true);
    	clickById(buttonId);
    	waitForAjaxCompletion();
    	checkButton(buttonId, true, ajaxSingle, immediate, true, true, true, true);
    	
    	buttonId = parentId + "b2";
    	ajaxSingle = false;
    	immediate = false;
    	writeStatus("Click button 2");
    	setValidation(false);
    	clickById(buttonId);
    	waitForAjaxCompletion();
    	checkButton(buttonId, false, ajaxSingle, immediate, false, false, false, false);
    	
    	buttonId = parentId + "b3";
    	ajaxSingle = false;
    	immediate = true;
    	writeStatus("Click button 3");
    	setValidation(true);
    	clickById(buttonId);
    	waitForAjaxCompletion();
    	checkButton(buttonId, true, ajaxSingle, immediate, true, true, true, false);
    	
    	buttonId = parentId + "b4";
    	ajaxSingle = true;
    	immediate = true;
    	writeStatus("Click button 4");
    	setValidation(false);
    	clickById(buttonId);
    	waitForAjaxCompletion();
    	checkButton(buttonId, true, ajaxSingle, immediate, true, true, true, false);
    	
      
    }
    
    private void setValidation(boolean success) {
    	StringBuffer buffer = new StringBuffer("setValidation(");
    	buffer.append(success);
    	buffer.append(");");
    	runScript(buffer.toString());
    }
    
    private void checkButton(String id, boolean testData, boolean ajaxSingle, boolean immediate, boolean testAction, boolean testActionListener, boolean testFListener, boolean testInput) {
    	writeStatus("Checking button...");
    	StringBuffer buffer = new StringBuffer("checkButton('");
    	buffer.append(id);
    	buffer.append("',");
    	buffer.append(testData);
    	buffer.append(",");
    	buffer.append(testAction);
    	buffer.append(",");
    	buffer.append(testActionListener);
    	buffer.append(",");
    	buffer.append(testFListener);
    	buffer.append(",");
    	buffer.append(testInput);
    	buffer.append(");");
    	String result = runScript(buffer.toString());
    	if (result != null && result.length() > 0) {
    		Assert.fail("<a4j:commandButton> [ajaxSingle="+ajaxSingle+" ; immediate="+immediate+"] test failure caused by " + result);
    	}
    	
    }

	public String getTestUrl() {
		return "/faces/pages/ajaxCommandButton/ajaxButtonTest.xhtml";
	}

}
