package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class PanelMenuTest  extends SeleniumTestBase implements RichSeleniumTest {
	
	public PanelMenuTest() {
		super("http", "localhost", serverPort);
	}
	
	/**
     * This method are invoking before selenium tests started
     */
	@BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
    	super.startSelenium(browser);
    }
    
    /**
     * This method are invoking after selenium tests completed
     */
	@AfterTest
    public void stopSelenium() {
    	super.stopSelenium();
    }
    
	@Test
    public void testPanelMenuComponent() throws Exception {
        
        _testPanelMenuComponent(Templates.SIMPLE);
        _testPanelMenuComponent(Templates.DATATABLE);
        _testPanelMenuComponent(Templates.MODALPANEL);
        
        _testPanelMenuComponentSingleMode(Templates.SIMPLE);
        _testPanelMenuComponentSingleMode(Templates.DATATABLE);
        _testPanelMenuComponentSingleMode(Templates.MODALPANEL);

        _testPanelMenuItemAction(Templates.SIMPLE);
        _testPanelMenuItemAction(Templates.DATATABLE);
        _testPanelMenuItemAction(Templates.MODALPANEL);
   
    }
	
	private void _testPanelMenuComponent(Templates template) {
		renderPage(getTestUrl(), template);
		String parentId = getParentId() + "_form:";
		
		String inputId = parentId + "_value";
        String outputId = parentId + "_value2";
        
        String ajaxGroup = "tablehide" + parentId + "pGroup1";
        String serverGroup = "tablehide" + parentId + "pGroup2";
        String clientGroup = "tablehide" + parentId + "pGroup3";
        String disabledGroup = "tablehide" + parentId + "pGroup4";
        
        String ajaxItem = "tablehide" + parentId + "pItem1";
        String serverItem = "tablehide" + parentId + "pItem2";
        String clientItem = "tablehide" + parentId + "pItem3";
        String disabledItem = "tablehide" + parentId + "pItem4";
        
        writeStatus("Click on ajax group");
        clickById(ajaxGroup);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1");
        AssertTextEquals(outputId, "1");
        Assert.assertTrue(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on server group");
    	clickCommandAndWait(serverGroup);
    	AssertValueEquals(inputId, "pGroup2");
        AssertTextEquals(outputId, "2");
        Assert.assertTrue(isVisibleById(ajaxItem));
        Assert.assertTrue(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on ajax group");
        clickById(ajaxGroup);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1");
        AssertTextEquals(outputId, "3");
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertTrue(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on server group");
        clickCommandAndWait(serverGroup);
        
    	AssertValueEquals(inputId, "pGroup2");
        AssertTextEquals(outputId, "4");
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on client group");
        clickById(clientGroup);
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertTrue(isVisibleById(clientItem));
        
        writeStatus("Click on client group");
        clickById(clientGroup);
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on disabled client group");
        clickById(disabledGroup);
        Assert.assertFalse(isVisibleById(disabledItem));
	}
	
	private void _testPanelMenuComponentSingleMode(Templates template) {
		renderPage(getTestUrl(), template);
		String parentId = getParentId() + "_form:";
		
		String inputId = parentId + "_value";
        String outputId = parentId + "_value2";
        
        String ajaxGroup = "tablehide" + parentId + "pGroup1_single";
        String serverGroup = "tablehide" + parentId + "pGroup2_single";
        String clientGroup = "tablehide" + parentId + "pGroup3_single";
        String disabledGroup = "tablehide" + parentId + "pGroup4_single";
        
        String ajaxItem = "tablehide" + parentId + "pItem1_single";
        String serverItem = "tablehide" + parentId + "pItem2_single";
        String clientItem = "tablehide" + parentId + "pItem3_single";
        String disabledItem = "tablehide" + parentId + "pItem4_single";
        
        writeStatus("Click on ajax group");
        clickById(ajaxGroup);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1_single");
        AssertTextEquals(outputId, "1");
        Assert.assertTrue(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on server group");
        clickCommandAndWait(serverGroup);
    	AssertValueEquals(inputId, "pGroup2_single");
        AssertTextEquals(outputId, "2");
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertTrue(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on ajax group");
        clickById(ajaxGroup);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1_single");
        AssertTextEquals(outputId, "3");
        Assert.assertTrue(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on server group");
        clickCommandAndWait(serverGroup);
    	AssertValueEquals(inputId, "pGroup2_single");
        AssertTextEquals(outputId, "4");
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertTrue(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on client group");
        clickById(clientGroup);
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertTrue(isVisibleById(clientItem));
        
        writeStatus("Click on client group");
        clickById(clientGroup);
        Assert.assertFalse(isVisibleById(ajaxItem));
        Assert.assertFalse(isVisibleById(serverItem));
        Assert.assertFalse(isVisibleById(clientItem));
        
        writeStatus("Click on disabled client group");
        clickById(disabledGroup);
        Assert.assertFalse(isVisibleById(disabledItem));
	}
	
	private void _testPanelMenuItemAction(Templates template) {
		renderPage(getTestUrl(), template);
		String parentId = getParentId() + "_form:";
		
		String inputId = parentId + "_value";
        String outputId = parentId + "_value2";
		
		String group = "tablehide" + parentId + "pGroup1";
		String groupSingle = "tablehide" + parentId + "pGroup1_single";
	    
		String item = "tablehide" + parentId + "pItem1";
        String itemSingle = "tablehide" + parentId + "pItem1_single";
        
        writeStatus("Click on ajax group");
        clickById(group);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1");
        
    	writeStatus("Click on ajax group");
        clickById(groupSingle);
    	waitForAjaxCompletion();
    	AssertValueEquals(inputId, "pGroup1_single");
        
    	writeStatus("Click on server group");
    	clickCommandAndWait(item);
    	AssertValueEquals(inputId, "pItem1");
        AssertTextEquals(outputId, "1");
        
        writeStatus("Click on server group");
        clickCommandAndWait(itemSingle);
    	AssertValueEquals(inputId, "pItem1_single");
        AssertTextEquals(outputId, "2");
	}
	
    public String getTestUrl() {
		return "/faces/pages/panelMenu/panelMenuTest.xhtml";
	}

}
