/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class SpacerTest  extends SeleniumTestBase implements RichSeleniumTest {
	
	public SpacerTest() {
		super("http", "localhost", serverPort);
	}

	/**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
    	super.startSelenium(browser);
    }
    
    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
    	super.stopSelenium();
    }
    
    @Test
    public void testSimpleTogglePanelComponent() throws Exception {
    	_testSimpleTogglePanelComponent(Templates.SIMPLE);
    	_testSimpleTogglePanelComponent(Templates.DATATABLE);
    	_testSimpleTogglePanelComponent(Templates.MODALPANEL);
    }
    
    private void _testSimpleTogglePanelComponent(Templates template) {
    	renderPage(getTestUrl(), template);
		String parentId = getParentId();
    	String spacerId = parentId + "sp";
    	
    	writeStatus("Check width ");
    	assertWidth(1 + 2, spacerId);

    	writeStatus("Check height ");
    	assertHeight(5 + 2, spacerId);
    	
    	writeStatus("Check styleClass ");
    	assertStyleClass("sp", spacerId);
    	
    	writeStatus("Check style ");
    	assertStyleClass("border: solid 1px green;", spacerId);
   }
    
    public String getTestUrl() {
		return "/faces/pages/spacer/spacerTest.xhtml";
	}

	private void assertWidth(int width, String spacerId) {
		StringBuffer script = new StringBuffer(" document.getElementById('");
		script.append(spacerId);
		script.append("').offsetWidth");
		
		String w = runScript(script.toString());
		Assert.assertEquals(w, String.valueOf(width));
	}
	
	private void assertHeight(int height, String spacerId) {
		StringBuffer script = new StringBuffer(" document.getElementById('");
		script.append(spacerId);
		script.append("').offsetHeight");
		
		String h = runScript(script.toString());
		Assert.assertEquals(h, String.valueOf(height));
	}
	
	private void assertStyleClass(String styleClass, String spacerId) {
		StringBuffer script = new StringBuffer(" document.getElementById('");
		script.append(spacerId);
		script.append("').className");
		
		String sc = runScript(script.toString());
		if (sc != null) {
			Assert.assertTrue(sc.endsWith("sp"));
		} else {
			Assert.fail("ClassName is null");
		}
	}
	private void assertStyle(String style, String spacerId) {
		StringBuffer script = new StringBuffer(" document.getElementById('");
		script.append(spacerId);
		script.append("').style");
		
		String s = runScript(script.toString());
		Assert.assertEquals(s, style);
	}
}
